import sys,os
sys.path.append(os.getcwd())

import logging
from typing import Dict, List, Any, Optional
import asyncio
import datetime
from psycopg import sql


from pgmanage.asyncpg.connect import AsyncPgPool

# 设置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# import platform
# if platform.system() == 'Windows':
#     import asyncio
#     from asyncio import WindowsSelectorEventLoopPolicy
#     asyncio.set_event_loop_policy(WindowsSelectorEventLoopPolicy())

class AsyncPgExec:
    def __init__(self, conn_str: str = None,min_size=0,max_size=20,timeout=180):
        """
        初始化数据库连接配置。
        
        :param conn_str: 数据库连接字符串 'dbname=shop_data user=postgres password=xxxxx host=127.0.0.1 port=5432'
        :param min_size: 连接池最小连接数，默认为0
        :param max_size: 连接池最大连接数，默认为20
        :param timeout: 连接超时时间，默认为180秒
        """
        self.conn_str = conn_str
        self.pool = None
        self.async_pool = AsyncPgPool(conn_str,min_size=min_size,max_size=max_size,timeout=timeout)

    async def __aenter__(self):
        """异步上下文管理器入口"""
        self.pool = await self.async_pool.__aenter__()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器出口"""
        await self.async_pool.__aexit__(exc_type, exc_val, exc_tb)

    async def _exec_query(self, query: str, params: Optional[List[Any]] = None) -> bool:
        """执行SQL查询，并处理异常"""
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                try:
                    if params:
                        await cur.execute(query, params)
                    else:
                        await cur.execute(query)
                    await conn.commit()
                    return True
                except Exception as e:
                    await conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise

    async def exec_query(self, query: str, params: Optional[List[Any]] = None) -> List[Dict[str, Any]]:
        """执行SQL查询，返回查询数据列表字典"""
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                try:
                    if params:
                        await cur.execute(query, params)
                    else:
                        await cur.execute(query)
                    await conn.commit()
                    column_names = [desc[0] for desc in cur.description]
                    results = await cur.fetchall()
                    result_dicts = [dict(zip(column_names, row)) for row in results]
                    return result_dicts
                except Exception as e:
                    await conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise

    async def exec_query_rowcount(self, query: str, params: Optional[List[Any]] = None) -> int:
        """执行SQL查询，返回处理的数据行数"""
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                try:
                    if params:
                        await cur.execute(query, params)
                    else:
                        await cur.execute(query)
                    await conn.commit()
                    return cur.rowcount
                except Exception as e:
                    await conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise

    def infer_data_type(self, value: Any) -> str:
        """
        根据传入的值推断数据类型。
        
        :param value: 要推断的数据值
        :return: 推断出的数据类型
        """
        # print(value,type(value))
        if isinstance(value, int) and not isinstance(value, bool):
            return 'INT'
        elif isinstance(value, float):
            return 'FLOAT'
        elif isinstance(value, bool):
            return 'BOOLEAN'
        elif isinstance(value, str):
            return 'VARCHAR'
        elif isinstance(value, list):
            return 'VARCHAR[]'  # PostgreSQL 数组类型
        elif isinstance(value, dict):
            return 'JSONB'  # PostgreSQL JSONB 类型
        elif isinstance(value, (datetime.date, datetime.datetime)):
            return 'TIMESTAMP'
        else:
            return 'VARCHAR'  # 默认类型

    async def check_table(self, schema: str, table: str) -> bool:
        """检查表是否存在"""
        # query = "SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_schema = %s AND table_name = %s);"

        # 使用 sql 模块构建安全的SQL查询
        query = sql.SQL("SELECT EXISTS (SELECT 1 FROM information_schema.tables WHERE table_schema = {schema} AND table_name = {table})").format(
            schema=sql.Literal(schema),
            table=sql.Literal(table)
        )

        async with self.pool.connection() as conn:
            # print(query.as_string(conn))   # 输出SQL查询
            async with conn.cursor() as cur:
                # await cur.execute(query, (schema, table))
                await cur.execute(query)
                exists = (await cur.fetchone())[0]
                if not exists:
                    logger.warning(f"表 '{schema}.{table}' 不存在")
                return exists

    async def create_table(self, schema: str, table: str, columns: Dict[str, str], primary_key: Optional[str] = None) -> None:
        """
        创建新表。
        
        :param schema: 模式名
        :param table: 表名
        :param columns: 列定义的字典，键为列名，值为列类型
        :param primary_key: 主键列名，默认为None
        """
        # cols_sql = ', '.join([f"{col} {ctype}" for col, ctype in columns.items()])
        # pk_sql = f", PRIMARY KEY ({primary_key})" if primary_key else ""
        # create_sql = f"CREATE TABLE {schema}.{table} ({cols_sql}{pk_sql});"

        # 构建列定义部分
        cols_sql = sql.SQL(', ').join(
            sql.SQL("{col} {ctype}").format(
                col=sql.Identifier(col),
                ctype=sql.SQL(ctype)
            ) for col, ctype in columns.items()
        )

        # 如果有主键，则添加PRIMARY KEY约束
        pk_sql = sql.SQL(", PRIMARY KEY ({pk})").format(pk=sql.Identifier(primary_key)) if primary_key else sql.SQL("")

        # 构建完整的CREATE TABLE语句
        create_sql = sql.SQL("CREATE TABLE {schema}.{table} ({cols}{pk});").format(
            schema=sql.Identifier(schema),
            table=sql.Identifier(table),
            cols=cols_sql,
            pk=pk_sql
        )

        await self._exec_query(create_sql)
        logger.info(f"创建表 '{schema}.{table}' 成功")

    async def get_columns(self, schema: str, table: str) -> Dict[str, str]:
        """
        获取表的所有列及其数据类型。
        
        :param schema: 模式名
        :param table: 表名
        :return: 字典，键为列名，值为数据类型
        """
        query = "SELECT column_name, data_type FROM information_schema.columns WHERE table_schema = %s AND table_name = %s;"
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                await cur.execute(query, (schema, table))
                return {row[0]: row[1] for row in await cur.fetchall()}

    async def add_column(self, schema: str, table: str, column: str, column_type: str) -> None:
        """
        向表中添加一列。
        
        :param schema: 模式名
        :param table: 表名
        :param column: 新列名
        :param column_type: 新列的数据类型
        """
        # add_col_sql = f"ALTER TABLE {schema}.{table} ADD COLUMN {column} {column_type};"

        # 使用 sql 模块构建安全的SQL查询
        add_col_sql = sql.SQL("ALTER TABLE {schema}.{table} ADD COLUMN {column} {column_type};").format(
            schema=sql.Identifier(schema),
            table=sql.Identifier(table),
            column=sql.Identifier(column),
            column_type=sql.SQL(column_type)  # 注意：这里假设 column_type 是一个有效的 SQL 类型名称
        )

        await self._exec_query(add_col_sql)
        logger.info(f"向表 '{schema}.{table}' 添加列 '{column}',类型 '{column_type}' 成功")

    async def upsert_batch(self, schema, table, batch, conflict_target, do_update_set, start_index, end_index):
        """
        执行单个批次的异步插入或更新
        :param schema: 模式名
        :param table: 表名
        :param batch: 要插入或更新的数据批次
        :param conflict_target: 冲突目标，例如：ON CONFLICT (id) DO UPDATE SET
        :param do_update_set: 更新设置，例如：id = EXCLUDED.id, name = EXCLUDED.name
        :param start_index: 批次起始索引
        :param end_index: 批次结束索引
        :return: 批次中更新的记录数
        """
        if not batch:
            return 0

        # columns = ', '.join(batch[0].keys())
        columns = sql.SQL(", ").join(sql.Identifier(col) for col in batch[0].keys())
        # placeholders = ', '.join(['(%s)' % ', '.join(['%s'] * len(batch[0]))] * len(batch))

        # 构建批量插入的占位符
        placeholders = []
        for record in batch:
            placeholders.append(sql.SQL("({})").format(sql.SQL(", ").join([sql.Placeholder()] * len(record))))
        placeholders_sql = sql.SQL(", ").join(placeholders)

        
        values = []
        for record in batch:
            values.extend(record[col] for col in batch[0].keys())

        # sql_query = """
        # INSERT INTO {schema}.{table} ({columns}) VALUES {placeholders} 
        # ON CONFLICT {conflict_target} {do_update_set};
        # """

        # 构建SQL查询
        if do_update_set==None:
            sql_query = sql.SQL("""
                INSERT INTO {schema}.{table} ({columns}) VALUES {placeholders};
            """).format(
                schema = sql.Identifier(schema),
                table = sql.Identifier(table),
                columns=columns,
                placeholders=placeholders_sql,  # 注意：这里可能需要根据实际情况调整如何插入占位符
            )
        else:
            sql_query = sql.SQL("""
                INSERT INTO {schema}.{table} ({columns}) VALUES {placeholders} 
                ON CONFLICT {conflict_target} {do_update_set};
            """).format(
                schema = sql.Identifier(schema),
                table = sql.Identifier(table),
                columns=columns,
                placeholders=placeholders_sql,  # 注意：这里可能需要根据实际情况调整如何插入占位符
                conflict_target=conflict_target,
                do_update_set=do_update_set
            )
            
        
        async with self.pool.connection() as conn:
            try:
                async with conn.cursor() as cur:
                    await cur.execute(sql_query, values)
                    await conn.commit()
                    logger.info(f"成功处理第 {start_index + 1} 到 {end_index} 条记录，共 {cur.rowcount} 条")
                    return cur.rowcount
            except Exception as e:
                await conn.rollback()
                logger.error(f"处理第 {start_index + 1} 到 {end_index} 条记录时发生错误: {e}")
                logger.error(f"错误行号: {e.__traceback__.tb_lineno}")
                raise

    async def upsert_data(self, schema: str, table: str, data: List[Dict[str, Any]], 
                         batch_size: int = None, update: bool = False, 
                         create: bool = False, add_columns: bool = False,
                         skipPK=False,only_column: bool = False) -> int:
        """
        异步批量插入或更新数据
        :param schema: 模式名
        :param table: 表名
        :param data: 数据列表，每个元素为一个字典，键为列名，值为列值
        :param batch_size: 每批次处理的记录数，默认为65535/列数
        :param update: 是否更新已存在的记录，默认为False
        :param create: 如果表不存在，是否创建表，默认为False
        :param add_columns: 如果列不存在，是否添加列，默认为False
        :param only_column: 是否只更新数据库中存在的列，默认为False
        :param skipPK: 是否跳过主键检查，默认为False不跳过
        :return: 处理的总记录数
        """
        
        if not isinstance(data, list) or not data:
            raise ValueError("无效的数据列表输入")

        if not await self.check_table(schema, table):
            if create:
                columns = {key: self.infer_data_type(value) 
                         for key, value in (data[0] if data else {}).items()}
                await self.create_table(schema, table, columns)
            else:
                raise ValueError(f"表 '{schema}.{table}' 不存在")

        db_columns = await self.get_columns(schema, table)  # 数据库中的列
        all_columns = set().union(*(d.keys() for d in data)) # data中所有的列
        missing_columns = all_columns.difference(db_columns) # data中有 数据库中不存在的列

        if missing_columns:
            if add_columns:
                for col in missing_columns:
                    sample_value = next((d[col] for d in data if col in d), None)
                    col_type = self.infer_data_type(sample_value)
                    await self.add_column(schema, table, col, col_type)
                    db_columns[col] = col_type
            elif only_column:
                filter_data = ({k: d[k] for k in db_columns.keys() if k in d} for d in data)
                data = list(filter_data)
            else:
                raise ValueError(f"表 '{schema}.{table}' 中缺少列: {', '.join(missing_columns)}")

        batch_size_max = int(65535/len(data[0]))
        if not batch_size or batch_size > batch_size_max:
            batch_size = batch_size_max
            logger.info(f"batch_size 最大可设置为 {batch_size}")

        if not skipPK:
            primary_keys = await self.get_primary_keys(schema, table)
            if not primary_keys:
                raise ValueError("没有找到主键,未保证数据一致性暂停写入")

            # conflict_target = '({})'.format(', '.join(primary_keys))
            # do_update_set = "DO UPDATE SET " + ', '.join(
            #     [f"{col} = EXCLUDED.{col}" for col in data[0].keys() 
            #      if col not in primary_keys]) if update else "DO NOTHING"

            # 构建 CONFLICT 目标
            conflict_target = sql.SQL('({})').format(
                sql.SQL(', ').join(map(sql.Identifier, primary_keys))
            )

            # 构建 DO UPDATE SET 子句
            if update:
                do_update_set = sql.SQL("DO UPDATE SET {updates}").format(
                    updates=sql.SQL(', ').join(
                        sql.SQL("{col} = EXCLUDED.{col}").format(
                            col=sql.Identifier(col)
                        )
                        for col in data[0].keys() if col not in primary_keys
                    )
                )
            else:
                do_update_set = sql.SQL("DO NOTHING")
        else:
            conflict_target = None
            do_update_set=None

        total_count = 0
        tasks = []
        
        for i in range(0, len(data), batch_size):
            batch = data[i:i + batch_size]
            end_index = min(i + batch_size - 1, len(data) - 1)
            task = self.upsert_batch(schema, table, batch, conflict_target, 
                                   do_update_set, i, end_index)
            tasks.append(task)

        results = await asyncio.gather(*tasks, return_exceptions=True)
        for result in results:
            if isinstance(result, Exception):
                logger.error(f"批处理失败: {result}")
                raise result
            else:
                total_count += result

        return total_count

    async def get_primary_keys(self, schema: str, table: str) -> List[str]:
        """
        获取给定表的主键列表。
        
        :param schema: 模式名
        :param table: 表名
        :return: 主键列表
        """
        query = """
        SELECT a.attname
        FROM   pg_index i
        JOIN   pg_attribute a ON a.attnum = ANY(i.indkey) AND a.attrelid = i.indrelid
        WHERE  i.indrelid = %s::regclass
        AND    i.indisprimary;
        """
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                await cur.execute(query, (f'"{schema}"."{table}"',))
                primary_keys = [row[0] for row in await cur.fetchall()]
                if not primary_keys:
                    logger.error(f"表 '{schema}.{table}' 没有主键")
                    raise ValueError(f"表 '{schema}.{table}' 没有主键")
                return primary_keys






import sys,os
sys.path.append(os.getcwd())

## 同步使用示例

from pgmanage import PgPool

dbm = PgPool()
with dbm.pool.connection() as conn:
    with conn.cursor() as cur:
        cur.execute("SELECT 1")
        data = cur.fetchone()
        print("查询结果:",data)



# 异步使用示例

import asyncio
from pgmanage import AsyncPgPool

# async def test():
#     dbm =await DbmanagerAsync.connect()
#     async with dbm.pool.connection() as conn:
#         async with conn.cursor() as cur:
#             await cur.execute("SELECT 1")
#             data = await cur.fetchone()
#             print(data)



# if __name__ == "__main__":
#     asyncio.run(test())



import sys,os
sys.path.append(os.getcwd())

from pgmanage.syncpg.connect import PgPool
from pgmanage.syncpg.dbexec import PgExec
from pgmanage.asyncpg.connect import AsyncPgPool
from pgmanage.asyncpg.dbexec import AsyncPgExec
__all__ = [
    'PgPool',
    'PgExec',
    'AsyncPgPool',
    'AsyncPgExec',
    ]


# test.py

import os
import sys
sys.path.append(os.getcwd())  # 确保可以从当前目录导入模块

# 设置日志
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


import asyncio
from pgmanage import AsyncPgPool
async def main():
    try:
        # 初始化连接池
        pool = await AsyncPgPool.initialize()

        # 使用连接池执行数据库操作
        async with pool.connection() as conn:
            async with conn.cursor() as cur:
                await cur.execute("SELECT 1")
                rows = await cur.fetchall()
                for row in rows:
                    print("查询结果:",row)
    except Exception as e:
        logger.error(f"发生错误: {e}")
        raise

if __name__ == "__main__":
    asyncio.run(main())


import logging
from psycopg_pool import AsyncConnectionPool
from contextlib import asynccontextmanager

logger = logging.getLogger(__name__)

class PgPool:
    _instance = None
    _pool = None
    _conn_str = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    @classmethod
    def init_pool(cls, conn_str: str):
        """初始化连接字符串"""
        cls._conn_str = conn_str

    @classmethod
    @asynccontextmanager
    async def get_pool(cls):
        """获取连接池的异步上下文管理器"""
        if cls._conn_str is None:
            raise ValueError("连接池未初始化，请先调用 init_pool(conn_str)")
            
        try:
            async with AsyncConnectionPool(cls._conn_str) as pool:
                yield pool
        except Exception as e:
            logger.error(f"连接池操作失败: {e}")
            raise 




import atexit
import os
from psycopg_pool import ConnectionPool
# 设置日志
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


import platform
if platform.system() == 'Windows':
    import asyncio
    from asyncio import WindowsSelectorEventLoopPolicy
    asyncio.set_event_loop_policy(WindowsSelectorEventLoopPolicy())

class PgPool:
    def __init__(self, conn_str: str = None,min_size=0,max_size=20,timeout=180):
        if conn_str is None:
            conn_str = os.environ.get("PGLINK_URL")
            if conn_str is None:
                print("示例:dbname=mydb user=myuser password=mypassword host=myhost port=5432")
                raise ValueError("Database connection string not provided and PGLINK_URL environment variable not set.")
        
        try:
            self.pool = ConnectionPool(conninfo=conn_str,min_size=min_size,max_size=max_size,timeout=timeout)
        except Exception as e:
            raise RuntimeError(f"Failed to initialize database connection pool: {e}")
        
        atexit.register(self.close_pool)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close_pool()

    def close_pool(self):
        if hasattr(self, 'pool') and self.pool is not None:
            try:
                self.pool.close()
                logger.info("运行完毕已自动关闭连接池")
            finally:
                self.pool = None



import sys,os
sys.path.append(os.getcwd())


import logging
import asyncio
from pgmanage import AsyncPgExec

# 设置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


# 使用示例
async def main():
    import os
    DBURI = os.environ.get("PGLINK_URL")
    
    # 方式1：使用上下文管理器（推荐）
    async with AsyncPgExec(DBURI) as dbmange:
        # 生成测试数据
        data_to_bulk_insert = [
            {'id': f'{i}', 'name': f'name{i}88', 'age': i % 100, 
             'email': f'email{i}@example.com', 'fasle': True,'测试':f'ce-{i}'}
            for i in range(100000)
        ]

        result = await dbmange.upsert_data(
            'data_test', 'table_tEst', data_to_bulk_insert,
            batch_size=10000, update=True, create=True, add_columns=False,only_column=True
        )
        
        logger.info(f"总共成功插入或更新了 {result} 条记录")


    # 方式2：直接使用（程序结束时会自动关闭连接池）
    # 生成测试数据
    data_to_bulk_insert = [
        {'id': f'{i}', 'name': f'name{i}88', 'age': i % 100, 
         'email': f'email{i}@example.com', 'fasle': True,'平均支付_签收时长(秒)':90,'签收时长(秒)':33}
        for i in range(100000)
    ]
    db_exec = AsyncPgExec(DBURI)
    db_exec.pool = await db_exec.async_pool.initialize()
    result = await db_exec.upsert_data(
        'data_test', 'table_tEst', data_to_bulk_insert,
        batch_size=None, update=True, create=True, add_columns=False
    )
    logger.info(f"总共成功插入或更新了 {result} 条记录")
    # 不需要手动关闭，atexit 会处理

if __name__ == "__main__":
    asyncio.run(main())







import logging
from datetime import datetime, date
from typing import Dict, List, Any, Optional
from psycopg import sql
import concurrent.futures
from pgmanage import PgPool

# 设置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class PgExec():
    def __init__(self, conn_str: str=None,min_size=0,max_size=20,timeout=180):
        """
        初始化数据库连接池。
        
        :param conn_str: 数据库连接字符串 'dbname=shop_data user=postgres password=xxxxx host=127.0.0.1 port=5432'
        :param min_size: 连接池最小连接数，默认为0
        :param max_size: 连接池最大连接数，默认为20
        :param timeout: 连接超时时间，默认为180秒
        """
        self.pool = PgPool(conn_str,min_size=min_size,max_size=max_size,timeout=timeout).pool

    def _exec_query(self, query: str, params: Optional[List[Any]] = None) -> bool:
        """执行SQL查询，并处理异常"""
        with self.pool.connection() as conn:
            with conn.cursor() as cur:
                try:
                    if params:
                        cur.execute(query, params)
                    else:
                        cur.execute(query)
                    conn.commit()
                    return True
                except Exception as e:
                    conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise
    def exec_query(self, query: str, params: Optional[List[Any]] = None) -> List[Dict[str, Any]]:
        """执行SQL查询，返回查询数据列表字典"""
        with self.pool.connection() as conn:
            with conn.cursor() as cur:
                try:
                    if params:
                        cur.execute(query, params)
                    else:
                        cur.execute(query)
                    conn.commit()
                    column_names = [desc[0] for desc in cur.description]
                    results = cur.fetchall()
                    result_dicts = [dict(zip(column_names, row)) for row in results]
                    return result_dicts
                except Exception as e:
                    conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise
    def exec_query_rowcount(self, query: str, params: Optional[List[Any]] = None) -> int:
        """执行SQL查询，返回处理的数据行数"""
        with self.pool.connection() as conn:
            with conn.cursor() as cur:
                try:
                    if params:
                        cur.execute(query, params)
                    else:
                        cur.execute(query)
                    conn.commit()
                    return cur.rowcount
                except Exception as e:
                    conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise
    def infer_data_type(self, value: Any) -> str:
            """
            根据传入的值推断数据类型。
            
            :param value: 要推断的数据值
            :return: 推断出的数据类型
            """
            # print(value,type(value))
            if isinstance(value, int) and not isinstance(value, bool):
                return 'INT'
            elif isinstance(value, float):
                return 'FLOAT'
            elif isinstance(value, bool):
                return 'BOOLEAN'
            elif isinstance(value, str):
                return 'VARCHAR'
            elif isinstance(value, list):
                return 'VARCHAR[]'  # PostgreSQL 数组类型
            elif isinstance(value, dict):
                return 'JSONB'  # PostgreSQL JSONB 类型
            elif isinstance(value, (date, datetime)):
                return 'TIMESTAMP'
            else:
                return 'VARCHAR'  # 默认类型
            
    def check_table(self, schema: str, table: str) -> bool:
        """
        检查表是否存在。
        
        :param schema: 模式名
        :param table: 表名
        :return: 如果存在返回True，否则False
        """
        # query = "SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_schema = %s AND table_name = %s);"

        # 使用 sql 模块构建安全的SQL查询
        query = sql.SQL("SELECT EXISTS (SELECT 1 FROM information_schema.tables WHERE table_schema = {schema} AND table_name = {table})").format(
            schema=sql.Literal(schema),
            table=sql.Literal(table)
        )

        with self.pool.connection() as conn:
            # print(query.as_string(conn))   # 输出SQL查询
            with conn.cursor() as cur:
                # cur.execute(query, (schema, table))
                cur.execute(query)
                exists = cur.fetchone()[0]
                if not exists:
                    logger.warning(f"表 '{schema}.{table}' 不存在")
                return exists

    def create_table(self, schema: str, table: str, columns: Dict[str, str], primary_key: Optional[str] = None) -> None:
        """
        创建新表。
        
        :param schema: 模式名
        :param table: 表名
        :param columns: 列定义的字典，键为列名，值为列类型
        :param primary_key: 主键列名，默认为None
        """
        # cols_sql = ', '.join([f"{col} {ctype}" for col, ctype in columns.items()])
        # pk_sql = f", PRIMARY KEY ({primary_key})" if primary_key else ""
        # create_sql = f"CREATE TABLE {schema}.{table} ({cols_sql}{pk_sql});"

        # 构建列定义部分
        cols_sql = sql.SQL(', ').join(
            sql.SQL("{col} {ctype}").format(
                col=sql.Identifier(col),
                ctype=sql.SQL(ctype)
            ) for col, ctype in columns.items()
        )

        # 如果有主键，则添加PRIMARY KEY约束
        pk_sql = sql.SQL(", PRIMARY KEY ({pk})").format(pk=sql.Identifier(primary_key)) if primary_key else sql.SQL("")

        # 构建完整的CREATE TABLE语句
        create_sql = sql.SQL("CREATE TABLE {schema}.{table} ({cols}{pk});").format(
            schema=sql.Identifier(schema),
            table=sql.Identifier(table),
            cols=cols_sql,
            pk=pk_sql
        )


        self._exec_query(create_sql)
        logger.info(f"创建表 '{schema}.{table}' 成功")

    def get_columns(self, schema: str, table: str) -> Dict[str, str]:
        """
        获取表的所有列及其数据类型。
        
        :param schema: 模式名
        :param table: 表名
        :return: 字典，键为列名，值为数据类型
        """
        query = "SELECT column_name, data_type FROM information_schema.columns WHERE table_schema = %s AND table_name = %s;"
        with self.pool.connection() as conn:
            with conn.cursor() as cur:
                cur.execute(query, (schema, table))
                return {row[0]: row[1] for row in cur.fetchall()}

    def add_column(self, schema: str, table: str, column: str, column_type: str) -> None:
        """
        向表中添加一列。
        
        :param schema: 模式名
        :param table: 表名
        :param column: 新列名
        :param column_type: 新列的数据类型
        """
        # add_col_sql = f"ALTER TABLE {schema}.{table} ADD COLUMN {column} {column_type};"

        # 使用 sql 模块构建安全的SQL查询
        add_col_sql = sql.SQL("ALTER TABLE {schema}.{table} ADD COLUMN {column} {column_type};").format(
            schema=sql.Identifier(schema),
            table=sql.Identifier(table),
            column=sql.Identifier(column),
            column_type=sql.SQL(column_type)  # 注意：这里假设 column_type 是一个有效的 SQL 类型名称
        )
        self._exec_query(add_col_sql)
        logger.info(f"向表 '{schema}.{table}' 添加列 '{column}',类型 '{column_type}' 成功")

    def upsert_batch(self, schema:str, table:str, batch, conflict_target, do_update_set, start_index, end_index)->int:
        """
        执行单个批次的插入或更新
        :param schema: 模式名
        :param table: 表名
        :param batch: 批次数据
        :param conflict_target: 冲突目标
        :param do_update_set: 更新设置
        :param start_index: 起始索引
        :param end_index: 结束索引
        :return: 插入或更新的记录数
        """
        if not batch:
            return 0
        
        # columns = ', '.join(batch[0].keys())
        columns = sql.SQL(", ").join(sql.Identifier(col) for col in batch[0].keys())
        # placeholders = ', '.join(['(%s)' % ', '.join(['%s'] * len(batch[0]))] * len(batch))

        # 构建批量插入的占位符
        placeholders = []
        for record in batch:
            placeholders.append(sql.SQL("({})").format(sql.SQL(", ").join([sql.Placeholder()] * len(record))))
        placeholders_sql = sql.SQL(", ").join(placeholders)
        
        values = []
        for record in batch:
            values.extend(record[col] for col in batch[0].keys())

        # sql_query = f"""
        # INSERT INTO {schema}.{table} ({columns}) VALUES {placeholders} 
        # ON CONFLICT {conflict_target} {do_update_set};
        # """

        # 构建SQL查询
        if do_update_set == None :
            sql_query = sql.SQL("""
                INSERT INTO {schema}.{table} ({columns}) VALUES {placeholders};
            """).format(
                schema = sql.Identifier(schema),
                table = sql.Identifier(table),
                columns=columns,
                placeholders=placeholders_sql,  # 注意：这里可能需要根据实际情况调整如何插入占位符
            )
        else:
            sql_query = sql.SQL("""
                INSERT INTO {schema}.{table} ({columns}) VALUES {placeholders} 
                ON CONFLICT {conflict_target} {do_update_set};
            """).format(
                schema = sql.Identifier(schema),
                table = sql.Identifier(table),
                columns=columns,
                placeholders=placeholders_sql,  # 注意：这里可能需要根据实际情况调整如何插入占位符
                conflict_target=conflict_target,
                do_update_set=do_update_set
            )

        result = 0
        with self.pool.connection() as conn:
            try:
                with conn.cursor() as cur:
                    cur.execute(sql_query, values)
                    conn.commit()
                    result = cur.rowcount
                    logger.info(f"成功处理第 {start_index + 1} 到 {end_index} 条记录，共 {cur.rowcount} 条,写入:{result}条")
            except Exception as e:
                conn.rollback()
                logger.error(f"处理第 {start_index + 1} 到 {end_index} 条记录时发生错误: {e}")
                logger.error(f"错误行号: {e.__traceback__.tb_lineno}")
                raise
        return result
        
    def upsert_data(self, schema: str, table: str, data: List[Dict[str, Any]], batch_size: int = None, 
            update: bool = False, create: bool = False, add_columns: bool = False, only_column: bool = False
            ,skipPK=False, max_workers: int = 5) -> int:
        """
        使用多线程分批插入或更新数据到指定表中。
        :param schema: 模式名
        :param table: 表名
        :param data: 数据列表，每个元素为一个字典，键为列名，值为列值
        :param batch_size: 每批次插入或更新的记录数，默认为65535/列数
        :param update: 是否允许更新，默认为False
        :param create: 是否允许创建，默认为False
        :param add_columns: 是否允许添加新列，默认为False
        :param only_column: 是否只更新数据库中已有的列，默认为False
        :param max_workers: 线程池的最大工作线程数，默认为5
        :param skipPK: 是否跳过主键检查，默认为False不跳过
        :return: 总的成功插入和更新的记录数
        """
        if not isinstance(data, list) or not data:
            raise ValueError("无效的数据列表输入")

        total_records = len(data)
        inserted_updated_count = 0

        # 检查表是否存在并根据参数决定是否创建
        if not self.check_table(schema, table):
            if create:
                columns = {key: self.infer_data_type(value) for key, value in (data[0] if data else {}).items()}  # 把第一行数据的值类型作为字段类型
                self.create_table(schema, table, columns, primary_key='id' if 'id' in columns else None)
            else:
                raise ValueError(f"表 '{schema}.{table}' 不存在")

        db_columns = self.get_columns(schema, table)
        all_columns = set().union(*(d.keys() for d in data)) # data中所有的列
        missing_columns = all_columns.difference(db_columns) # data中有 数据库中不存在的列

        if missing_columns:
            if add_columns:
                for col in missing_columns:
                    sample_value = next((d[col] for d in data if col in d), None)
                    col_type = self.infer_data_type(sample_value)
                    self.add_column(schema, table, col, col_type)
                    db_columns[col] = col_type
            elif only_column:
                filter_data = ({k: d[k] for k in db_columns.keys() if k in d} for d in data)
                data = list(filter_data)
            else:
                raise ValueError(f"表 '{schema}.{table}' 中缺少列: {', '.join(missing_columns)}")


        batch_size_max = int(65535/len(data[0]))
        if not batch_size or batch_size > batch_size_max:
            batch_size = batch_size_max
            logger.info(f"batch_size 最大可设置为 {batch_size}")

        # 如果检查主键
        if not skipPK:  
            primary_keys = self.get_primary_keys(schema, table)
            if not primary_keys:
                raise ValueError("没有找到主键,未保证数据一致性暂停写入")

            # conflict_target = '({})'.format(', '.join(primary_keys)) if primary_keys else ''
            # do_update_set = "DO UPDATE SET " + ', '.join([f"{col} = EXCLUDED.{col}" for col in data[0].keys() if col not in primary_keys]) if update else "DO NOTHING"

            # 构建 CONFLICT 目标
            conflict_target = sql.SQL('({})').format(
                sql.SQL(', ').join(map(sql.Identifier, primary_keys))
            )

            # 构建 DO UPDATE SET 子句
            if update:
                do_update_set = sql.SQL("DO UPDATE SET {updates}").format(
                    updates=sql.SQL(', ').join(
                        sql.SQL("{col} = EXCLUDED.{col}").format(
                            col=sql.Identifier(col)
                        )
                        for col in data[0].keys() if col not in primary_keys
                    )
                )
            else:
                do_update_set = sql.SQL("DO NOTHING")
        else:
            conflict_target = None
            do_update_set = None
        # 创建一个线程池
        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
            futures = []

            # 提交每个批次的任务给线程池
            for i in range(0, total_records, batch_size):
                start_index = i
                end_index = min(i + batch_size - 1, total_records - 1)
                batch = data[i:i + batch_size]
                future = executor.submit(
                    self.upsert_batch, schema, table, batch, 
                    conflict_target, do_update_set,start_index,end_index)
                futures.append(future)

            # 收集所有线程的结果
            for future in concurrent.futures.as_completed(futures):
                result = future.result()
                if isinstance(result, Exception):
                    logger.error(f"批处理失败: {result}")
                    raise result
                else:
                    inserted_updated_count += result

        return inserted_updated_count

    def get_primary_keys(self, schema: str, table: str) -> List[str]:
        """
        获取给定表的主键列表。
        
        :param schema: 模式名
        :param table: 表名
        :return: 主键列表
        """
        query = """
        SELECT a.attname
        FROM   pg_index i
        JOIN   pg_attribute a ON a.attnum = ANY(i.indkey) AND a.attrelid = i.indrelid
        WHERE  i.indrelid = %s::regclass
        AND    i.indisprimary;
        """
        with self.pool.connection() as conn:
            with conn.cursor() as cur:
                cur.execute(query, (f'"{schema}"."{table}"',))
                primary_keys = [row[0] for row in cur.fetchall()]
                if not primary_keys:
                    logger.warning(f"表 '{schema}.{table}' 没有主键")
                    raise ValueError(f"表 '{schema}.{table}' 没有主键")
                return primary_keys

# 使用示例
if __name__ == "__main__":
    db_handler = PgExec()

    # 假设有一个包含大量记录的数据列表
    data_to_bulk_insert = [
        {'id1': '1','id2': 11, 'id3': 11,'name': 'Alice', 'age': 30, 'email': 'alice@example.com'},
        {'id1': '2','id2': 12, 'id3': 11, 'name': 'Bob', 'age': 25, 'email': 'bob@example.com'},
        # ... 更多数据
    ]
    
    # 生成10万行的测试数据
    data_to_bulk_insert = []
    for i in range(100000):
        data_to_bulk_insert.append({'id': f'{i}', 'name': f'name{i}88', 'age': i % 100, 'email': f'email{i}@example.com','fasle': True})

    result = db_handler.upsert_data('data_test', 'table_test', data_to_bulk_insert, batch_size=10000, 
                                        update=True, create=True, add_columns=True)
    logger.info(f"总共成功插入或更新了 {result} 条记录")




import os
import sys
sys.path.append(os.getcwd())

# 设置日志
import logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

from pgmanage import PgExec

def main():

    # 生成10万行的测试数据
    data_to_bulk_insert = []
    for i in range(100000):
        data_to_bulk_insert.append({'id': f'{i}', 'name2': f'name{i}88','name2': f'name{i}88', 'age': i % 100, 'email': f'email{i}@example.com','fasle': True})


    # 创建数据库管理器实例
    db_mgr = PgExec()

    # 写入数据，主键冲突时更新所有非主键列 upsert_data
    rows_inserted = db_mgr.upsert_data(
        schema='data_test', 
        table='table_tEst', 
        data=data_to_bulk_insert,
        batch_size=10000,
        update=True,
        create=True,
        add_columns=True,
        max_workers=10,
        only_column=True
    )
    logger.info(f"总共插入 {rows_inserted} 行")

    # # 执行 SQL 查询
    # query = """SELECT * FROM "data_test"."table_tEst" LIMIT 10;"""
    # result = db_mgr.exec_query(query)
    # for row in result:
    #     logger.info(row)
        
    # 删除数据
    # query = """delete  FROM "data_test"."table_tEst";"""
    # result = db_mgr.exec_query_rowcount(query)
    # logger.info(f"删除结果:{result}")

if __name__ == '__main__':
    main()


import sys,os
sys.path.append(os.getcwd())

## 同步使用示例
from pgmanage import PgPool

dbm = PgPool()
with dbm.pool.connection() as conn:
    with conn.cursor() as cur:
        cur.execute("SELECT 1")
        data = cur.fetchone()
        print("查询结果:",data)

with dbm.pool.connection() as conn:
    with conn.cursor() as cur:
        cur.execute("SELECT 1")
        data = cur.fetchone()
        print("查询结果:",data)
        
with dbm.pool.connection() as conn:
    with conn.cursor() as cur:
        cur.execute("SELECT 1")
        data = cur.fetchone()
        print("查询结果:",data)


import logging
from typing import Dict, List, Any, Optional
import asyncio
import psycopg
from psycopg_pool import AsyncConnectionPool
import datetime
import platform

# 设置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

if platform.system() == 'Windows':
    import asyncio
    from asyncio import WindowsSelectorEventLoopPolicy
    asyncio.set_event_loop_policy(WindowsSelectorEventLoopPolicy())

class AsyncPgtool:
    def __init__(self, conn_str: str = None):
        """
        初始化数据库连接配置。
        
        :param conn_str: 数据库连接字符串
        """
        self.conn_str = conn_str

    async def __aenter__(self):
        """异步上下文管理器入口"""
        # 直接使用 AsyncConnectionPool 的上下文管理器
        self._pool_ctx = AsyncConnectionPool(self.conn_str)
        self.pool = await self._pool_ctx.__aenter__()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器出口"""
        if hasattr(self, '_pool_ctx'):
            await self._pool_ctx.__aexit__(exc_type, exc_val, exc_tb)

    async def _exec_query(self, query: str, params: Optional[List[Any]] = None) -> bool:
        """执行SQL查询，并处理异常"""
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                try:
                    if params:
                        await cur.execute(query, params)
                    else:
                        await cur.execute(query)
                    await conn.commit()
                    return True
                except Exception as e:
                    await conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise

    async def exec_query(self, query: str, params: Optional[List[Any]] = None) -> List[Dict[str, Any]]:
        """执行SQL查询，返回查询数据列表字典"""
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                try:
                    if params:
                        await cur.execute(query, params)
                    else:
                        await cur.execute(query)
                    await conn.commit()
                    column_names = [desc[0] for desc in cur.description]
                    results = await cur.fetchall()
                    result_dicts = [dict(zip(column_names, row)) for row in results]
                    return result_dicts
                except Exception as e:
                    await conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise

    async def exec_query_rowcount(self, query: str, params: Optional[List[Any]] = None) -> int:
        """执行SQL查询，返回处理的数据行数"""
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                try:
                    if params:
                        await cur.execute(query, params)
                    else:
                        await cur.execute(query)
                    await conn.commit()
                    return cur.rowcount
                except Exception as e:
                    await conn.rollback()
                    logger.error(f"查询执行失败: {e}")
                    raise

    def infer_data_type(self, value: Any) -> str:
        """
        根据传入的值推断数据类型。
        
        :param value: 要推断的数据值
        :return: 推断出的数据类型
        """
        print(value,type(value))
        if isinstance(value, int) and not isinstance(value, bool):
            return 'INT'
        elif isinstance(value, float):
            return 'FLOAT'
        elif isinstance(value, bool):
            return 'BOOLEAN'
        elif isinstance(value, str):
            return 'VARCHAR'
        elif isinstance(value, list):
            return 'VARCHAR[]'  # PostgreSQL 数组类型
        elif isinstance(value, dict):
            return 'JSONB'  # PostgreSQL JSONB 类型
        elif isinstance(value, (datetime.date, datetime.datetime)):
            return 'TIMESTAMP'
        else:
            return 'VARCHAR'  # 默认类型

    async def check_table(self, schema: str, table: str) -> bool:
        """检查表是否存在"""
        query = "SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_schema = %s AND table_name = %s);"
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                await cur.execute(query, (schema, table))
                exists = (await cur.fetchone())[0]
                if not exists:
                    logger.warning(f"表 '{schema}.{table}' 不存在")
                return exists

    async def create_table(self, schema: str, table: str, columns: Dict[str, str], primary_key: Optional[str] = None) -> None:
        """
        创建新表。
        
        :param schema: 模式名
        :param table: 表名
        :param columns: 列定义的字典，键为列名，值为列类型
        :param primary_key: 主键列名，默认为None
        """
        cols_sql = ', '.join([f"{col} {ctype}" for col, ctype in columns.items()])
        pk_sql = f", PRIMARY KEY ({primary_key})" if primary_key else ""
        create_sql = f"CREATE TABLE {schema}.{table} ({cols_sql}{pk_sql});"
        await self._exec_query(create_sql)
        logger.info(f"创建表 '{schema}.{table}' 成功")

    async def get_columns(self, schema: str, table: str) -> Dict[str, str]:
        """
        获取表的所有列及其数据类型。
        
        :param schema: 模式名
        :param table: 表名
        :return: 字典，键为列名，值为数据类型
        """
        query = "SELECT column_name, data_type FROM information_schema.columns WHERE table_schema = %s AND table_name = %s;"
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                await cur.execute(query, (schema, table))
                return {row[0]: row[1] for row in await cur.fetchall()}

    async def add_column(self, schema: str, table: str, column: str, column_type: str) -> None:
        """
        向表中添加一列。
        
        :param schema: 模式名
        :param table: 表名
        :param column: 新列名
        :param column_type: 新列的数据类型
        """
        add_col_sql = f"ALTER TABLE {schema}.{table} ADD COLUMN {column} {column_type};"
        await self._exec_query(add_col_sql)
        logger.info(f"向表 '{schema}.{table}' 添加列 '{column}',类型 '{column_type}' 成功")

    async def upsert_batch(self, schema, table, batch, conflict_target, do_update_set, start_index, end_index):
        """执行单个批次的异步插入或更新"""
        if not batch:
            return 0

        columns = ', '.join(batch[0].keys())
        placeholders = ', '.join(['(%s)' % ', '.join(['%s'] * len(batch[0]))] * len(batch))
        
        values = []
        for record in batch:
            values.extend(record[col] for col in batch[0].keys())

        sql = f"""
        INSERT INTO {schema}.{table} ({columns}) VALUES {placeholders} 
        ON CONFLICT {conflict_target} {do_update_set};
        """
        
        try:
            async with self.pool.connection() as conn:
                async with conn.cursor() as cur:
                    await cur.execute(sql, values)
                    await conn.commit()
                    logger.info(f"成功处理第 {start_index + 1} 到 {end_index} 条记录，共 {len(batch)} 条")
                    return len(batch)
        except Exception as e:
            logger.error(f"处理第 {start_index + 1} 到 {end_index} 条记录时发生错误: {e}")
            raise

    async def upsert_data(self, schema: str, table: str, data: List[Dict[str, Any]], 
                         batch_size: int = 1000, update: bool = False, 
                         create: bool = False, add_columns: bool = False) -> int:
        """异步批量插入或更新数据"""
        if not isinstance(data, list) or not data:
            raise ValueError("无效的数据列表输入")

        if not await self.check_table(schema, table):
            if create:
                columns = {key: self.infer_data_type(value) 
                         for key, value in (data[0] if data else {}).items()}
                await self.create_table(schema, table, columns)
            else:
                raise ValueError(f"表 '{schema}.{table}' 不存在")

        existing_columns = await self.get_columns(schema, table)

        if add_columns:
            all_columns = set().union(*(d.keys() for d in data))
            for col in all_columns:
                if col not in existing_columns:
                    sample_value = next((d[col] for d in data if col in d), None)
                    col_type = self.infer_data_type(sample_value)
                    await self.add_column(schema, table, col, col_type)

        primary_keys = await self.get_primary_keys(schema, table)
        if not primary_keys:
            raise ValueError("没有找到主键,未保证数据一致性暂停写入")

        conflict_target = '({})'.format(', '.join(primary_keys))
        do_update_set = "DO UPDATE SET " + ', '.join(
            [f"{col} = EXCLUDED.{col}" for col in data[0].keys() 
             if col not in primary_keys]) if update else "DO NOTHING"

        total_count = 0
        tasks = []
        
        for i in range(0, len(data), batch_size):
            batch = data[i:i + batch_size]
            end_index = min(i + batch_size - 1, len(data) - 1)
            task = self.upsert_batch(schema, table, batch, conflict_target, 
                                   do_update_set, i, end_index)
            tasks.append(task)

        results = await asyncio.gather(*tasks, return_exceptions=True)
        for result in results:
            if isinstance(result, Exception):
                logger.error(f"批处理失败: {result}")
            else:
                total_count += result

        return total_count

    async def get_primary_keys(self, schema: str, table: str) -> List[str]:
        """
        获取给定表的主键列表。
        
        :param schema: 模式名
        :param table: 表名
        :return: 主键列表
        """
        query = """
        SELECT a.attname
        FROM   pg_index i
        JOIN   pg_attribute a ON a.attnum = ANY(i.indkey) AND a.attrelid = i.indrelid
        WHERE  i.indrelid = %s::regclass
        AND    i.indisprimary;
        """
        async with self.pool.connection() as conn:
            async with conn.cursor() as cur:
                await cur.execute(query, (f"{schema}.{table}",))
                primary_keys = [row[0] for row in await cur.fetchall()]
                if not primary_keys:
                    logger.warning(f"表 '{schema}.{table}' 没有主键")
                    raise ValueError(f"表 '{schema}.{table}' 没有主键")
                return primary_keys

# 使用示例
async def main():
    import os
    DBURI = os.environ.get("PGLINK_URL")
    async with AsyncPgtool(DBURI) as db_handler:
        # 生成测试数据
        data_to_bulk_insert = [
            {'id': f'{i}', 'name': f'name{i}88', 'age': i % 100, 
             'email': f'email{i}@example.com', 'fasle': True}
            for i in range(100000)
        ]

        result = await db_handler.upsert_data(
            'data_test', 'table_test', data_to_bulk_insert,
            batch_size=10000, update=True, create=True, add_columns=True
        )
        
        logger.info(f"总共成功插入或更新了 {result} 条记录")

if __name__ == "__main__":
    asyncio.run(main())





import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pgmanage",
    version="1.0.13",
    author="manji",
    author_email="pnsm@qq.com",                         # 作者邮箱
    description="pgsql数据库工具",                            # 模块简介
    long_description=long_description,                  # 模块详细介绍
    long_description_content_type="text/markdown",      # 模块详细介绍格式
    url="https://gitee.com/manjim/pgmanage",
    packages=setuptools.find_packages(),                # 自动找到项目中导入的模块

    # 模块相关的元数据（更多描述信息）
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

    # 依赖模块
    install_requires=[
        'psycopg[binary,pool]',
    ],

    python_requires='>=3' ,                             # 运行的python版本
)


import logging
import os
import atexit
from typing import Optional
from psycopg_pool import AsyncConnectionPool

logger = logging.getLogger(__name__)

class AsyncPgPool:
    _instance = None
    _pool: Optional[AsyncConnectionPool] = None
    _pool_ctx = None
    _atexit_registered = False  # 添加标志以确保只注册一次

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super(AsyncPgPool, cls).__new__(cls)
        return cls._instance

    def __init__(self, conn_str: str = None,min_size=0,max_size=20,timeout=180):
        self.min_size = min_size
        self.max_size = max_size
        self.timeout = timeout
        if not hasattr(self, 'conn_str'):
            if conn_str is None:
                conn_str = os.environ.get("PGLINK_URL")
                if conn_str is None:
                    raise ValueError("数据库连接变量 PGLINK_URL 不存在")
            self.conn_str = conn_str
            # 确保 atexit 只注册一次
            if not AsyncPgPool._atexit_registered:
                atexit.register(self._cleanup)
                AsyncPgPool._atexit_registered = True

    async def __aenter__(self):
        """异步上下文管理器入口"""
        if self._pool is None:
            self._pool_ctx = AsyncConnectionPool(self.conn_str,min_size=self.min_size,max_size=self.max_size,timeout=self.timeout)
            self._pool = await self._pool_ctx.__aenter__()
            logger.info("数据库连接池已初始化")
        return self._pool

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器出口"""
        await self.close()

    @classmethod
    async def initialize(cls, conn_str: str=None):
        """初始化数据库连接池"""
        instance = cls(conn_str)
        return await instance.__aenter__()

    @classmethod
    def get_pool(cls) -> Optional[AsyncConnectionPool]:
        """获取连接池实例"""
        return cls._pool

    @classmethod
    async def close(cls):
        """关闭连接池"""
        if cls._pool_ctx is not None:
            await cls._pool_ctx.__aexit__(None, None, None)
            cls._pool = None
            cls._pool_ctx = None
            logger.info("数据库连接池已关闭")

    @classmethod
    def _cleanup(cls):
        """退出时的清理函数"""
        import asyncio
        try:
            try:
                loop = asyncio.get_event_loop()
            except RuntimeError:
                # 如果没有事件循环，创建一个新的
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)
            
            if loop.is_running():
                loop.create_task(cls.close())
            else:
                try:
                    loop.run_until_complete(cls.close())
                finally:
                    loop.close()
                    
        except Exception as e:
            logger.error(f"清理连接池时发生错误: {e}") 
